#include "ws2812.h"
#include "math.h"
//----------------------------------------------------------------------------
extern TIM_HandleTypeDef htim2;
//----------------------------------------------------------------------------
uint8_t BUF_DMA [ARRAY_LEN] = {0};
uint8_t rgb_temp[15][3];
RGB_t rgb_temp2[LED_COUNT];
HSV_t back_buf [LED_COUNT] = {0};
HSV_t back_buf_temp;
uint16_t DMA_BUF_TEMP[24];
//------------------------------------------------------------------
void ws2812_init(void)
{
  int i;
  for(i=DELAY_LEN;i<ARRAY_LEN;i++) BUF_DMA[i] = LOW;
}
//------------------------------------------------------------------
void ws2812_pixel_rgb_to_buf_dma(uint8_t Rpixel , uint8_t Gpixel, uint8_t Bpixel, uint16_t posX)
{
  volatile uint16_t i;
  for(i=0;i<8;i++)
  {
    if (BitIsSet(Rpixel,(7-i)) == 1)
    {
      BUF_DMA[DELAY_LEN+posX*24+i+8] = HIGH;
    }else
    {
      BUF_DMA[DELAY_LEN+posX*24+i+8] = LOW;
    }
    if (BitIsSet(Gpixel,(7-i)) == 1)
    {
      BUF_DMA[DELAY_LEN+posX*24+i+0] = HIGH;
    }else
    {
      BUF_DMA[DELAY_LEN+posX*24+i+0] = LOW;
    }
    if (BitIsSet(Bpixel,(7-i)) == 1)
    {
      BUF_DMA[DELAY_LEN+posX*24+i+16] = HIGH;
    }else
    {
      BUF_DMA[DELAY_LEN+posX*24+i+16] = LOW;
    }
  }
}
//------------------------------------------------------------------
void ws2812_hsv_to_rgb(void)
{
  uint8_t color[3];
  volatile uint16_t j;
  uint32_t base_V;
	for(j=0;j<LED_COUNT;j++)
  {
		if (back_buf[j].S == 0) 
    {
      color[0] = back_buf[j].V;
      color[1] = back_buf[j].V;
      color[2] = back_buf[j].V;
    }
		else
    {
      base_V = ((255 - back_buf[j].S) * back_buf[j].V) >> 8;
			switch (back_buf[j].H / 60)
      {
        case 0:
					color[0] = back_buf[j].V;
          color[1] = (((back_buf[j].V - base_V) * back_buf[j].H) / 60) + base_V;
          color[2] = base_V;
          break;
        case 1:
					color[0] = (((back_buf[j].V - base_V) * (60 - (back_buf[j].H % 60))) / 60) + base_V;
          color[1] = back_buf[j].V;
          color[2] = base_V;
          break;
        case 2:
          color[0] = base_V;
          color[1] = back_buf[j].V;
          color[2] = (((back_buf[j].V - base_V) * (back_buf[j].H % 60)) / 60) + base_V;
          break;
        case 3:
          color[0] = base_V;
          color[1] = (((back_buf[j].V - base_V) * (60 - (back_buf[j].H % 60))) / 60) + base_V;
          color[2] = back_buf[j].V;
          break;
        case 4:
          color[0] = (((back_buf[j].V - base_V) * (back_buf[j].H % 60)) / 60) + base_V;
          color[1] = base_V;
          color[2] = back_buf[j].V;
          break;
        case 5:
          color[0] = back_buf[j].V;
          color[1] = base_V;
          color[2] = (((back_buf[j].V - base_V) * (60 - (back_buf[j].H % 60))) / 60) + base_V;
          break;
      }
    }
    rgb_temp2[j].R = color[0];
    rgb_temp2[j].G = color[1];
    rgb_temp2[j].B = color[2];
  }
}
//------------------------------------------------------------------
void ws2812_rgb_copy_buf_to_dma(void)
{
  volatile uint16_t j;
  uint8_t i;
  for(j=0;j<LED_COUNT;j++)
  {
    for(i=0;i<8;i++)
    {
      if (BitIsSet(rgb_temp2[j].R,(7-i)) == 1)
      {
        BUF_DMA[DELAY_LEN+j*24+i+8] = HIGH;
      }else
      {
        BUF_DMA[DELAY_LEN+j*24+i+8] = LOW;
      }
      if (BitIsSet(rgb_temp2[j].G,(7-i)) == 1)
      {
        BUF_DMA[DELAY_LEN+j*24+i+0] = HIGH;
      }else
      {
        BUF_DMA[DELAY_LEN+j*24+i+0] = LOW;
      }
      if (BitIsSet(rgb_temp2[j].B,(7-i)) == 1)
      {
        BUF_DMA[DELAY_LEN+j*24+i+16] = HIGH;
      }else
      {
        BUF_DMA[DELAY_LEN+j*24+i+16] = LOW;
      }
    }
  }
}
//------------------------------------------------------------------
void ws2812_setValue(void)
{
  uint8_t n=0;
  for(n=0;n<20;n++)
  {
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[0][0], rgb_temp[0][1], rgb_temp[0][2], n*15);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[1][0], rgb_temp[1][1], rgb_temp[1][2], n*15+1);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[2][0], rgb_temp[2][1], rgb_temp[2][2], n*15+2);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[3][0], rgb_temp[3][1], rgb_temp[3][2], n*15+3);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[4][0], rgb_temp[4][1], rgb_temp[4][2], n*15+4);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[5][0], rgb_temp[5][1], rgb_temp[5][2], n*15+5);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[6][0], rgb_temp[6][1], rgb_temp[6][2], n*15+6);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[7][0], rgb_temp[7][1], rgb_temp[7][2], n*15+7);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[8][0], rgb_temp[8][1], rgb_temp[8][2], n*15+8);
    ws2812_pixel_rgb_to_buf_dma( rgb_temp[9][0], rgb_temp[9][1], rgb_temp[9][2], n*15+9);
    ws2812_pixel_rgb_to_buf_dma(rgb_temp[10][0],rgb_temp[10][1],rgb_temp[10][2],n*15+10);
    ws2812_pixel_rgb_to_buf_dma(rgb_temp[11][0],rgb_temp[11][1],rgb_temp[11][2],n*15+11);
    ws2812_pixel_rgb_to_buf_dma(rgb_temp[12][0],rgb_temp[12][1],rgb_temp[12][2],n*15+12);
    ws2812_pixel_rgb_to_buf_dma(rgb_temp[13][0],rgb_temp[13][1],rgb_temp[13][2],n*15+13);
    ws2812_pixel_rgb_to_buf_dma(rgb_temp[14][0],rgb_temp[14][1],rgb_temp[14][2],n*15+14);
  }
}
//------------------------------------------------------------------
void ws2812_light(void)
{
  HAL_TIM_PWM_Start_DMA(&htim2,TIM_CHANNEL_2,(uint32_t*)&BUF_DMA,ARRAY_LEN);
}
//------------------------------------------------------------------
void ws2812_test01(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT * n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
	for(j=0;j<LED_COUNT*nm;j++)
  {
    memcpy((void*)&back_buf_temp,(void*)&back_buf[0],4);
    for(i=0;i<(LED_COUNT-1);i++)
    {
      memcpy((void*)&back_buf[i],(void*)&back_buf[i+1],4);
    }
    memcpy((void*)&back_buf[LED_COUNT-1],(void*)&back_buf_temp,4);
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(20);
  }
}
//------------------------------------------------------------------
void ws2812_test02(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
  for(j=0;j<LED_COUNT*nm;j++)
  {
    memcpy((void*)&back_buf_temp,(void*)&back_buf[LED_COUNT-1],4);
    for(i=0;i<(LED_COUNT-1);i++)
    {
      memcpy((void*)&back_buf[LED_COUNT-i-1],(void*)&back_buf[LED_COUNT-i-2],4);
    }
    memcpy((void*)&back_buf[0],(void*)&back_buf_temp,4);
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(20);
  }
}
//----------------------------------------------------------------------------
void ws2812_test03(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint16_t k;
  uint8_t yy;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
  //ws2812_hsv_to_rgb();
  for(yy=0;yy<nm;yy++)
  {
    for(j=0;j<=32;j++)
    {
      for(i=0;i<LED_COUNT;i++)
      {
        k = BRIGHT * (32-j) / 32;
        back_buf[i].V = k;
      }
      ws2812_hsv_to_rgb();
      ws2812_rgb_copy_buf_to_dma();
      ws2812_light();
      HAL_Delay(10);
    }
    for(j=0;j<=32;j++)
    {
      for(i=0;i<LED_COUNT;i++)
      {
        k = BRIGHT * j / 32;
        back_buf[i].V = k;
      }
      ws2812_hsv_to_rgb();
      ws2812_rgb_copy_buf_to_dma();
      ws2812_light();
      HAL_Delay(10);
    }
  }
}
//----------------------------------------------------------------------------
void ws2812_test04(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint16_t k;
  uint8_t yy;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = 64;
  }
  ws2812_hsv_to_rgb();
  for(yy=0;yy<nm;yy++)
  {
    for(j=0;j<=64;j++)
    {
      for(i=0;i<LED_COUNT;i++)
      {
        if((i%(LED_COUNT/n*2)) < LED_COUNT/n)
        {
          k = BRIGHT * (64-j) / 64;
        }
        else
        {
          k = BRIGHT * j / 64;
        }
        back_buf[i].V = k;
      }
      ws2812_hsv_to_rgb();
      ws2812_rgb_copy_buf_to_dma();
      ws2812_light();
      HAL_Delay(10);
    }
    for(j=0;j<=64;j++)
    {
      for(i=0;i<LED_COUNT;i++)
      {
        if((i%(LED_COUNT/n*2)) < LED_COUNT/n)
        {
          k = BRIGHT * j / 64;
        }
        else
        {
          k = BRIGHT * (64-j) / 64;
        }
        back_buf[i].V = k;
      }
      ws2812_hsv_to_rgb();
      ws2812_rgb_copy_buf_to_dma();
      ws2812_light();
      HAL_Delay(10);
    }
  }
}
//----------------------------------------------------------------------------
void ws2812_test05(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint16_t k;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
  ws2812_hsv_to_rgb();
  for(i=0;i<LED_COUNT;i++) 
  {
    if((i%(LED_COUNT/n)) <= LED_COUNT/n/2)
    {
      k = BRIGHT * (i%(LED_COUNT/n)) / (LED_COUNT/n/2) * (i%(LED_COUNT/n)) / (LED_COUNT/n/2);
    }
    else
    {
      k = BRIGHT * (LED_COUNT/n - (i%(LED_COUNT/n))) / (LED_COUNT/n/2) * \
          (LED_COUNT/n - (i%(LED_COUNT/n))) / (LED_COUNT/n/2);
    }
    back_buf[i].V = k;			
  }
  for(j=0;j<LED_COUNT*nm;j++)
  {
    back_buf_temp.V = back_buf[0].V;
    for(i=0;i<(LED_COUNT-1);i++)
    {
      back_buf[i].V = back_buf[i+1].V;
    }
    back_buf[LED_COUNT-1].V = back_buf_temp.V;
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(20);
  }
}
//----------------------------------------------------------------------------
void ws2812_test06(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint16_t k;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
  ws2812_hsv_to_rgb();
  for(i=0;i<LED_COUNT;i++) 
  {
    if((i%(LED_COUNT/n)) <= LED_COUNT/n/2)
    {
      k = BRIGHT * (i%(LED_COUNT/n)) / (LED_COUNT/n/2) * (i%(LED_COUNT/n)) / (LED_COUNT/n/2);
    }
    else
    {
      k = BRIGHT * (LED_COUNT/n - (i%(LED_COUNT/n))) / (LED_COUNT/n/2) * \
          (LED_COUNT/n - (i%(LED_COUNT/n))) / (LED_COUNT/n/2);
    }
    back_buf[i].V = k;			
  }
  for(j=0;j<LED_COUNT*nm;j++)
  {
    back_buf_temp.V = back_buf[LED_COUNT-1].V;
    for(i=0;i<(LED_COUNT-1);i++)
    {
      back_buf[LED_COUNT-i-1].V = back_buf[LED_COUNT-i-2].V;
    }
    back_buf[0].V = back_buf_temp.V;
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(20);
  }
}
//----------------------------------------------------------------------------
void ws2812_test07(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint8_t k;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
  ws2812_hsv_to_rgb();
  for(j=0;j<(nm*2);j++)
  {
    for(i=0;i<LED_COUNT;i++) 
    {
      if(j%2==0)
      {
        k = BRIGHT;
      }
      else
      {
        k = 0;
      }
      back_buf[i].V = k;			
    }
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(150);
  }
}
//----------------------------------------------------------------------------
void ws2812_test08(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint8_t k;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
  ws2812_hsv_to_rgb();
  for(j=0;j<(nm*2);j++)
  {
    for(i=0;i<LED_COUNT;i++) 
    {
      if(j%2==0)
      {
        if((i%(LED_COUNT/n)) < LED_COUNT/n/2)
        {
          k = BRIGHT;
        }
        else
        {
          k = 0;
        }
      }
      else
      {
        if((i%(LED_COUNT/n)) < LED_COUNT/n/2)
        {
          k = 0;
        }
        else
        {
          k = BRIGHT;
        }
      }
      back_buf[i].V = k;			
    }
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(200);
  }
}
//----------------------------------------------------------------------------
void ws2812_test09(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint16_t ks,kv;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
  ws2812_hsv_to_rgb();
  for(i=0;i<LED_COUNT;i++) 
  {
    if((i%(LED_COUNT/n)) <= LED_COUNT/n/2)
    {
      ks = 255 * pow(i%(LED_COUNT/n),1.0f/3.0f) / pow(LED_COUNT/n/2,1.0f/3.0f);
    }
    else
    {
      ks = 255 * pow(LED_COUNT/n - (i%(LED_COUNT/n)),1.0f/3.0f) / pow(LED_COUNT/n/2,1.0f/3.0f);
    }
    back_buf[i].S = ks;
    kv = (rgb_temp2[i].R + rgb_temp2[i].G + rgb_temp2[i].B)/3;
    back_buf[i].V = kv;
  }
  for(j=0;j<LED_COUNT*nm;j++)
  {
    back_buf_temp.S = back_buf[LED_COUNT-1].S;
    back_buf_temp.V = back_buf[LED_COUNT-1].V;
    for(i=0;i<(LED_COUNT-1);i++)
    {
      back_buf[LED_COUNT-i-1].S = back_buf[LED_COUNT-i-2].S;
      back_buf[LED_COUNT-i-1].V = back_buf[LED_COUNT-i-2].V;
    }
    back_buf[0].S = back_buf_temp.S;
    back_buf[0].V = back_buf_temp.V;
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(20);
  }
}
//----------------------------------------------------------------------------
void ws2812_test10(uint8_t n, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint16_t ks,kv;
  for(i=0;i<LED_COUNT;i++)
  {
    back_buf[i].H = i * 360 / LED_COUNT* n % 360; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
  }
  ws2812_hsv_to_rgb();
  for(i=0;i<LED_COUNT;i++) 
  {
    if((i%(LED_COUNT/n)) <= LED_COUNT/n/2)
    {
      ks = 255 * pow(i%(LED_COUNT/n),1.0f/3.0f) / pow(LED_COUNT/n/2,1.0f/3.0f);
    }
    else
    {
      ks = 255 * pow(LED_COUNT/n - (i%(LED_COUNT/n)),1.0f/3.0f) / pow(LED_COUNT/n/2,1.0f/3.0f);
    }
    back_buf[i].S = ks;
    kv = (rgb_temp2[i].R + rgb_temp2[i].G + rgb_temp2[i].B)/3;
    back_buf[i].V = kv;
  }
  for(j=0;j<LED_COUNT*nm;j++)
  {
    back_buf_temp.S = back_buf[0].S;
    back_buf_temp.V = back_buf[0].V;
    for(i=0;i<(LED_COUNT-1);i++)
    {
      back_buf[i].V = back_buf[i+1].V;
      back_buf[i].S = back_buf[i+1].S;
    }
    back_buf[LED_COUNT-1].V = back_buf_temp.V;
    back_buf[LED_COUNT-1].S = back_buf_temp.S;
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(20);
  }	
}
//----------------------------------------------------------------------------
void ws2812_test11(uint8_t n, uint8_t col, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint32_t k;
  for(i=0;i<LED_COUNT;i++)
  {
    switch(col)
    {
      case 1:
        back_buf[i].H = 0; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 2:
        back_buf[i].H = 40; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 3:
        back_buf[i].H = 80; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 4:
        back_buf[i].H = 120; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 5:
        back_buf[i].H = 160; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 6:
        back_buf[i].H = 200; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 7:
        back_buf[i].H = 240; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 8:
        back_buf[i].H = 280; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 9:
        back_buf[i].H = 320; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
    }
  }
  ws2812_hsv_to_rgb();
  for(i=0;i<LED_COUNT;i++) 
  {
    if((i%(LED_COUNT/n)) <= LED_COUNT/n/2)
    {
      k = BRIGHT * (i%(LED_COUNT/n)) / (LED_COUNT/n/2) * (i%(LED_COUNT/n)) / (LED_COUNT/n/2);
    }
    else
    {
      k = BRIGHT * (LED_COUNT/n - (i%(LED_COUNT/n))) / (LED_COUNT/n/2) * \
          (LED_COUNT/n - (i%(LED_COUNT/n))) / (LED_COUNT/n/2);
    }
    back_buf[i].V = k;
  }
  for(j=0;j<LED_COUNT*nm;j++)
  {
    back_buf_temp.V = back_buf[0].V;
    for(i=0;i<(LED_COUNT-1);i++)
    {
      back_buf[i].V = back_buf[i+1].V;
    }
    back_buf[LED_COUNT-1].V = back_buf_temp.V;
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(20);
  }
}
//----------------------------------------------------------------------------
void ws2812_test12(uint8_t n, uint8_t col, uint8_t nm)
{
  uint16_t i;
  uint16_t j;
  uint32_t k;
  for(i=0;i<LED_COUNT;i++)
  {
    switch(col)
    {
      case 1:
        back_buf[i].H = 0; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 2:
        back_buf[i].H = 40; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 3:
        back_buf[i].H = 80; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 4:
        back_buf[i].H = 120; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 5:
        back_buf[i].H = 160; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 6:
        back_buf[i].H = 200; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 7:
        back_buf[i].H = 240; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 8:
        back_buf[i].H = 280; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
      case 9:
        back_buf[i].H = 320; back_buf[i].S = 255, back_buf[i].V = BRIGHT;
        break;
    }
  }
  ws2812_hsv_to_rgb();
  for(i=0;i<LED_COUNT;i++) 
  {
    if((i%(LED_COUNT/n)) <= LED_COUNT/n/2)
    {
      k = BRIGHT * (i%(LED_COUNT/n)) / (LED_COUNT/n/2) * (i%(LED_COUNT/n)) / (LED_COUNT/n/2);
    }
    else
    {
      k = BRIGHT * (LED_COUNT/n - (i%(LED_COUNT/n))) / (LED_COUNT/n/2) * \
          (LED_COUNT/n - (i%(LED_COUNT/n))) / (LED_COUNT/n/2);
    }
    back_buf[i].V = k;			
  }
  for(j=0;j<LED_COUNT*nm;j++)
  {
    back_buf_temp.V = back_buf[LED_COUNT-1].V;
    for(i=0;i<(LED_COUNT-1);i++)
    {
      back_buf[LED_COUNT-i-1].V = back_buf[LED_COUNT-i-2].V;
    }
    back_buf[0].V = back_buf_temp.V;
    ws2812_hsv_to_rgb();
    ws2812_rgb_copy_buf_to_dma();
    ws2812_light();
    HAL_Delay(20);
  }
}
//----------------------------------------------------------------------------
void ws2812_test13(void)
{
  uint16_t i;
  uint8_t j,jj;
  int k=0;
  for(jj=0;jj<40;jj++)
  {
    for(j=0;j<32;j++)
    {
      for(i=0;i<LED_COUNT;i++)
      {
        if(j<16) k=1000*(j)/16;
        else k=1000*(32-j)/16;
        if(i%10==0) ws2812_pixel_rgb_to_buf_dma(BRIGHT*k/1000,BRIGHT*(1000-k)/1000,0,i);
        if(i%10==1) ws2812_pixel_rgb_to_buf_dma(0,BRIGHT*k/1000,BRIGHT*(1000-k)/1000,i);
        if(i%10==2) ws2812_pixel_rgb_to_buf_dma(BRIGHT*k/1000,0,BRIGHT*(1000-k)/1000,i);
        if(i%10==3) ws2812_pixel_rgb_to_buf_dma(BRIGHT*(1000-k)/1000,BRIGHT*k/2000,255*k/2000,i);
        if(i%10==4) ws2812_pixel_rgb_to_buf_dma(BRIGHT*k/2000,BRIGHT*k/2000,BRIGHT*(1000-k)/1000,i);
        if(i%10==5) ws2812_pixel_rgb_to_buf_dma(BRIGHT*k/2000,BRIGHT*(1000-k)/1000,BRIGHT*k/2000,i);
        if(i%10==6) ws2812_pixel_rgb_to_buf_dma(BRIGHT*(1000-k)/2000,BRIGHT*(1000-k)/2000,BRIGHT*k/1000,i);
        if(i%10==7) ws2812_pixel_rgb_to_buf_dma(BRIGHT*k/1000,BRIGHT*(1000-k)/2000,BRIGHT*(1000-k)/2000,i);
        if(i%10==8) ws2812_pixel_rgb_to_buf_dma(BRIGHT*(1000-k)/2000,BRIGHT*k/1000,BRIGHT*(1000-k)/2000,i);
        if(i%10==9) ws2812_pixel_rgb_to_buf_dma(BRIGHT*(1000-k)/3000,BRIGHT*(1000-k)/1333,BRIGHT*k/2000,i);
      }
      ws2812_light();
      HAL_Delay(20);
    }
  }
}
//----------------------------------------------------------------------------
